@@ -1,6 +1,8 @@
 #include "mqtt_number.h"
 #include "esphome/core/log.h"
 
+#include "mqtt_const.h"
+
 #ifdef USE_MQTT
 #ifdef USE_NUMBER
 
@@ -15,7 +17,7 @@ MQTTNumberComponent::MQTTNumberComponent(Number *number) : MQTTComponent(), numb
 
 void MQTTNumberComponent::setup() {
   this->subscribe(this->get_command_topic_(), [this](const std::string &topic, const std::string &state) {
-    auto val = parse_float(state);
+    auto val = parse_number<float>(state);
     if (!val.has_value()) {
       ESP_LOGW(TAG, "Can't convert '%s' to number!", state.c_str());
       return;
@@ -35,12 +37,24 @@ void MQTTNumberComponent::dump_config() {
 std::string MQTTNumberComponent::component_type() const { return "number"; }
 const EntityBase *MQTTNumberComponent::get_entity() const { return this->number_; }
 
-void MQTTNumberComponent::send_discovery(JsonObject &root, mqtt::SendDiscoveryConfig &config) {
+void MQTTNumberComponent::send_discovery(JsonObject root, mqtt::SendDiscoveryConfig &config) {
   const auto &traits = number_->traits;
   // https://www.home-assistant.io/integrations/number.mqtt/
-  root["min"] = traits.get_min_value();
-  root["max"] = traits.get_max_value();
-  root["step"] = traits.get_step();
+  root[MQTT_MIN] = traits.get_min_value();
+  root[MQTT_MAX] = traits.get_max_value();
+  root[MQTT_STEP] = traits.get_step();
+  if (!this->number_->traits.get_unit_of_measurement().empty())
+    root[MQTT_UNIT_OF_MEASUREMENT] = this->number_->traits.get_unit_of_measurement();
+  switch (this->number_->traits.get_mode()) {
+    case NUMBER_MODE_AUTO:
+      break;
+    case NUMBER_MODE_BOX:
+      root[MQTT_MODE] = "box";
+      break;
+    case NUMBER_MODE_SLIDER:
+      root[MQTT_MODE] = "slider";
+      break;
+  }
 
   config.command_topic = true;
 }
