@@ -96,6 +96,7 @@ void SSD1306::setup() {
     case SSD1306_MODEL_64_48:
     case SSD1306_MODEL_64_32:
     case SH1106_MODEL_64_48:
+    case SH1107_MODEL_128_64:
     case SSD1305_MODEL_128_32:
     case SSD1305_MODEL_128_64:
       this->command(0x12);
@@ -111,7 +112,14 @@ void SSD1306::setup() {
 
   // Set V_COM (0xDB)
   this->command(SSD1306_COMMAND_SET_VCOM_DETECT);
-  this->command(0x00);
+  switch (this->model_) {
+    case SH1107_MODEL_128_64:
+      this->command(0x35);
+      break;
+    default:
+      this->command(0x00);
+      break;
+  }
 
   // Display output follow RAM (0xA4)
   this->command(SSD1306_COMMAND_DISPLAY_ALL_ON_RESUME);
@@ -198,6 +206,8 @@ void SSD1306::turn_off() {
 }
 int SSD1306::get_height_internal() {
   switch (this->model_) {
+    case SH1107_MODEL_128_64:
+      return 128;
     case SSD1306_MODEL_128_32:
     case SSD1306_MODEL_64_32:
     case SH1106_MODEL_128_32:
@@ -232,6 +242,7 @@ int SSD1306::get_width_internal() {
     case SSD1306_MODEL_64_48:
     case SSD1306_MODEL_64_32:
     case SH1106_MODEL_64_48:
+    case SH1107_MODEL_128_64:
       return 64;
     default:
       return 0;
@@ -289,6 +300,8 @@ const char *SSD1306::model_str_() {
       return "SH1106 96x16";
     case SH1106_MODEL_64_48:
       return "SH1106 64x48";
+    case SH1107_MODEL_128_64:
+      return "SH1107 128x64";
     case SSD1305_MODEL_128_32:
       return "SSD1305 128x32";
     case SSD1305_MODEL_128_64:
